@@ -1,7 +1,7 @@
 """Microsoft Internet Explorer cookie loading on Windows.
 
 Copyright 2002-2003 Johnny Lee <typo_pl@hotmail.com> (MSIE Perl code)
-Copyright 2002-2003 John J Lee <jjl@pobox.com> (The Python port)
+Copyright 2002-2006 John J Lee <jjl@pobox.com> (The Python port)
 
 This code is free software; you can redistribute it and/or modify it under
 the terms of the BSD License (see the file COPYING included with the
@@ -9,11 +9,14 @@ distribution).
 
 """
 
+# XXX names and comments are not great here
+
 import os, re, string, time, struct
 if os.name == "nt":
     import _winreg
 
-from _ClientCookie import CookieJar, Cookie, MISSING_FILENAME_TEXT
+from _ClientCookie import FileCookieJar, CookieJar, Cookie, \
+     MISSING_FILENAME_TEXT, LoadError
 from _Util import startswith
 from _Debug import getLogger
 debug = getLogger("ClientCookie").debug
@@ -54,44 +57,7 @@ def epoch_time_offset_from_win32_filetime(filetime):
 def binary_to_char(c): return "%02X" % ord(c)
 def binary_to_str(d): return string.join(map(binary_to_char, list(d)), "")
 
-class MSIECookieJar(CookieJar):
-    """
-    This class differs from CookieJar only in the format it uses to load cookies
-    from a file.
-
-    MSIECookieJar can read the cookie files of Microsoft Internet Explorer
-    (MSIE) for Windows version 5 on Windows NT and version 6 on Windows XP and
-    Windows 98.  Other configurations may also work, but are untested.  Saving
-    cookies in MSIE format is NOT supported.  If you save cookies, they'll be
-    in the usual Set-Cookie3 format, which you can read back in using an
-    instance of the plain old CookieJar class.  Don't save using the same
-    filename that you loaded cookies from, because you may succeed in
-    clobbering your MSIE cookies index file!
-
-    You should be able to have LWP share Internet Explorer's cookies like
-    this (note you need to supply a username to load_from_registry if you're on
-    Windows 9x or Windows ME):
-
-    cj = MSIECookieJar(delayload=1)
-    # find cookies index file in registry and load cookies from it
-    cj.load_from_registry()
-    opener = ClientCookie.build_opener(ClientCookie.HTTPCookieProcessor(cj))
-    response = opener.open("http://example.com/")
-
-    Iterating over a delayloaded MSIECookieJar instance will not cause any
-    cookies to be read from disk.  To force reading of all cookies from disk,
-    call read_all_cookies.  Note that the following methods iterate over self:
-    clear_temporary_cookies, clear_expired_cookies, __len__, __repr__, __str__
-    and as_string.
-
-    Additional methods:
-
-    load_from_registry(ignore_discard=False, ignore_expires=False,
-                       username=None)
-    load_cookie_data(filename, ignore_discard=False, ignore_expires=False)
-    read_all_cookies()
-
-    """
+class MSIEBase:
     magic_re = re.compile(r"Client UrlCache MMF Ver \d\.\d.*")
     padding = "\x0d\xf0\xad\x0b"
 
@@ -104,32 +70,9 @@ class MSIECookieJar(CookieJar):
                r"\currentversion\explorer\shell folders"
     reg_key = "Cookies"
 
-    def __init__(self, *args, **kwds):
-        apply(CookieJar.__init__, (self,)+args, kwds)
+    def __init__(self):
         self._delayload_domains = {}
 
-    def set_cookie(self, cookie):
-        if self.delayload:
-            self._delayload_domain(cookie.domain)
-        CookieJar.set_cookie(self, cookie)
-
-    def _cookies_for_domain(self, domain, request, unverifiable):
-        debug("Checking %s for cookies to return", domain)
-        if not self.policy.domain_return_ok(domain, request, unverifiable):
-            return []
-
-        if self.delayload:
-            self._delayload_domain(domain)
-
-        return CookieJar._cookies_for_domain(
-            self, domain, request, unverifiable)
-
-    def read_all_cookies(self):
-        """Eagerly read in all cookies."""
-        if self.delayload:
-            for domain in self._delayload_domains.keys():
-                self._delayload_domain(domain)
-
     def _delayload_domain(self, domain):
         # if necessary, lazily load cookies for this domain
         delayload_info = self._delayload_domains.get(domain)
@@ -138,7 +81,7 @@ class MSIECookieJar(CookieJar):
             try:
                 self.load_cookie_data(cookie_file,
                                       ignore_discard, ignore_expires)
-            except IOError:
+            except (LoadError, IOError):
                 debug("error reading cookie file, skipping: %s", cookie_file)
             else:
                 del self._delayload_domains[domain]
@@ -258,26 +201,6 @@ class MSIECookieJar(CookieJar):
         filename = os.path.normpath(os.path.join(cookies_dir, "INDEX.DAT"))
         self.load(filename, ignore_discard, ignore_expires, username)
 
-    def load(self, filename, ignore_discard=False, ignore_expires=False,
-             username=None):
-        """Load cookies from an MSIE 'index.dat' cookies index file.
-
-        filename: full path to cookie index file
-        username: only required on win9x
-
-        """
-        if filename is None:
-            if self.filename is not None: filename = self.filename
-            else: raise ValueError(MISSING_FILENAME_TEXT)
-
-        index = open(filename, "rb")
-
-        try:
-            self._really_load(index, filename, ignore_discard, ignore_expires,
-                              username)
-        finally:
-            index.close()
-
     def _really_load(self, index, filename, ignore_discard, ignore_expires,
                      username):
         now = int(time.time())
@@ -289,7 +212,7 @@ class MSIECookieJar(CookieJar):
 
         data = index.read(256)
         if len(data) != 256:
-            raise IOError("%s file is too short" % filename)
+            raise LoadError("%s file is too short" % filename)
 
         # Cookies' index.dat file starts with 32 bytes of signature
         # followed by an offset to the first record, stored as a little-
@@ -299,7 +222,7 @@ class MSIECookieJar(CookieJar):
 
         # check that sig is valid
         if not self.magic_re.match(sig) or size != 0x4000:
-            raise IOError("%s ['%s' %s] does not seem to contain cookies" %
+            raise LoadError("%s ['%s' %s] does not seem to contain cookies" %
                           (str(filename), sig, size))
 
         # skip to start of first record
@@ -366,7 +289,7 @@ class MSIECookieJar(CookieJar):
                     try:
                         self.load_cookie_data(cookie_file,
                                               ignore_discard, ignore_expires)
-                    except IOError:
+                    except (LoadError, IOError):
                         debug("error reading cookie file, skipping: %s",
                               cookie_file)
                 else:
@@ -377,3 +300,94 @@ class MSIECookieJar(CookieJar):
 
                     self._delayload_domains[domain] = (
                         cookie_file, ignore_discard, ignore_expires)
+
+
+class MSIECookieJar(MSIEBase, FileCookieJar):
+    """FileCookieJar that reads from the Windows MSIE cookies database.
+
+    MSIECookieJar can read the cookie files of Microsoft Internet Explorer
+    (MSIE) for Windows version 5 on Windows NT and version 6 on Windows XP and
+    Windows 98.  Other configurations may also work, but are untested.  Saving
+    cookies in MSIE format is NOT supported.  If you save cookies, they'll be
+    in the usual Set-Cookie3 format, which you can read back in using an
+    instance of the plain old CookieJar class.  Don't save using the same
+    filename that you loaded cookies from, because you may succeed in
+    clobbering your MSIE cookies index file!
+
+    You should be able to have LWP share Internet Explorer's cookies like
+    this (note you need to supply a username to load_from_registry if you're on
+    Windows 9x or Windows ME):
+
+    cj = MSIECookieJar(delayload=1)
+    # find cookies index file in registry and load cookies from it
+    cj.load_from_registry()
+    opener = ClientCookie.build_opener(ClientCookie.HTTPCookieProcessor(cj))
+    response = opener.open("http://example.com/")
+
+    Iterating over a delayloaded MSIECookieJar instance will not cause any
+    cookies to be read from disk.  To force reading of all cookies from disk,
+    call read_all_cookies.  Note that the following methods iterate over self:
+    clear_temporary_cookies, clear_expired_cookies, __len__, __repr__, __str__
+    and as_string.
+
+    Additional methods:
+
+    load_from_registry(ignore_discard=False, ignore_expires=False,
+                       username=None)
+    load_cookie_data(filename, ignore_discard=False, ignore_expires=False)
+    read_all_cookies()
+
+    """
+    def __init__(self, filename=None, delayload=False, policy=None):
+        MSIEBase.__init__(self)
+        FileCookieJar.__init__(self, filename, delayload, policy)
+
+    def set_cookie(self, cookie):
+        if self.delayload:
+            self._delayload_domain(cookie.domain)
+        CookieJar.set_cookie(self, cookie)
+
+    def _cookies_for_request(self, request):
+        """Return a list of cookies to be returned to server."""
+        domains = self._cookies.copy()
+        domains.update(self._delayload_domains)
+        domains = domains.keys()
+
+        cookies = []
+        for domain in domains:
+            cookies.extend(self._cookies_for_domain(domain, request))
+        return cookies
+
+    def _cookies_for_domain(self, domain, request):
+        if not self._policy.domain_return_ok(domain, request):
+            return []
+        debug("Checking %s for cookies to return", domain)
+        if self.delayload:
+            self._delayload_domain(domain)
+        return CookieJar._cookies_for_domain(self, domain, request)
+
+    def read_all_cookies(self):
+        """Eagerly read in all cookies."""
+        if self.delayload:
+            for domain in self._delayload_domains.keys():
+                self._delayload_domain(domain)
+
+    def load(self, filename, ignore_discard=False, ignore_expires=False,
+             username=None):
+        """Load cookies from an MSIE 'index.dat' cookies index file.
+
+        filename: full path to cookie index file
+        username: only required on win9x
+
+        """
+        if filename is None:
+            if self.filename is not None: filename = self.filename
+            else: raise ValueError(MISSING_FILENAME_TEXT)
+
+        index = open(filename, "rb")
+
+        try:
+            self._really_load(index, filename, ignore_discard, ignore_expires,
+                              username)
+        finally:
+            index.close()
