@@ -1,6 +1,6 @@
 """Python backwards-compat., date/time routines, seekable file object wrapper.
 
- Copyright 2002-2003 John J Lee <jjl@pobox.com>
+ Copyright 2002-2006 John J Lee <jjl@pobox.com>
 
 This code is free software; you can redistribute it and/or modify it under
 the terms of the BSD License (see the file COPYING included with the
@@ -39,6 +39,15 @@ def compat_issubclass(obj, tuple_or_class):
         return False
     return issubclass(obj, tuple_or_class)
 
+def compat_isinstance(obj, tuple_or_class):
+    # for 2.1 and below
+    if type(tuple_or_class) == TupleType:
+        for klass in tuple_or_class:
+            if isinstance(obj, klass):
+                return True
+        return False
+    return isinstance(obj, tuple_or_class)
+
 def isstringlike(x):
     try: x+""
     except: return False
@@ -53,6 +62,8 @@ def isspace(string):
         if not SPACE_DICT.has_key(c): return False
     return True
 
+# this is here rather than in _HeadersUtil as it's just for
+# compatibility with old Python versions, rather than entirely new code
 def getheaders(msg, name):
     """Get all values for a header.
 
@@ -358,11 +369,18 @@ def iso2time(text):
     return _str2time(day, mon, yr, hr, min, sec, tz)
 
 
-
 # XXX Andrew Dalke kindly sent me a similar class in response to my request on
 # comp.lang.python, which I then proceeded to lose.  I wrote this class
 # instead, but I think he's released his code publicly since, could pinch the
 # tests from it, at least...
+
+# For testing seek_wrapper invariant (note that
+# test_urllib2.HandlerTest.test_seekable is expected to fail when this
+# invariant checking is turned on).  The invariant checking is done by module
+# ipdc, which is available here:
+# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/436834
+## from ipdbc import ContractBase
+## class seek_wrapper(ContractBase):
 class seek_wrapper:
     """Adds a seek method to a file object.
 
@@ -386,29 +404,49 @@ class seek_wrapper:
     # instance.  Seems to be some cStringIO.StringIO problem on 1.5.2 -- I
     # get a StringOobject, with no readlines method.
 
-    # Invariant: the end of the cache is always at the same place as the
-    # end of the wrapped file:
-    # self.wrapped.tell() == len(self._cache.getvalue())
+    # XXX Does this work sensibly in the face of exceptions raised by .read()
+    # / .readline()??
 
     def __init__(self, wrapped):
         self.wrapped = wrapped
         self.__have_readline = hasattr(self.wrapped, "readline")
         self.__cache = StringIO()
 
-    def __getattr__(self, name): return getattr(self.wrapped, name)
+    def invariant(self):
+        # The end of the cache is always at the same place as the end of the
+        # wrapped file.
+        return self.wrapped.tell() == len(self.__cache.getvalue())
+
+    def __getattr__(self, name):
+        wrapped = self.__dict__.get("wrapped")
+        if wrapped:
+            return getattr(wrapped, name)
+        return getattr(self.__class__, name)
 
     def seek(self, offset, whence=0):
+        assert whence in [0,1,2]
         # make sure we have read all data up to the point we are seeking to
         pos = self.__cache.tell()
-        if whence == 0:  # absolute
-            to_read = offset - pos
-        elif whence == 1:  # relative to current position
-            to_read = offset
-        elif whence == 2:  # relative to end of *wrapped* file
+
+        # how much data, if any, do we need to read?
+        if whence == 2:  # 2: relative to end of *wrapped* file
             # since we don't know yet where the end of that file is, we must
             # read everything
             to_read = None
-        if to_read is None or to_read >= 0:
+        else:
+            if whence == 0:  # 0: absolute
+                want = offset - pos
+            else:  # 1: relative to current position
+                want = offset
+            end = len(self.__cache.getvalue())
+            available = end - pos
+            if want <= available:
+                to_read = 0
+            else:
+                to_read = want - available
+
+        if to_read != 0:
+            self.__cache.seek(0, 2)
             if to_read is None:
                 self.__cache.write(self.wrapped.read())
             else:
@@ -417,11 +455,11 @@ class seek_wrapper:
 
         return self.__cache.seek(offset, whence)
 
+    def tell(self):
+        return self.__cache.tell()
+
     def read(self, size=-1):
         pos = self.__cache.tell()
-
-        self.__cache.seek(pos)
-
         end = len(self.__cache.getvalue())
         available = end - pos
 
@@ -483,7 +521,7 @@ class seek_wrapper:
                 (self.__class__.__name__, `id(self)`, `self.wrapped`))
 
     def close(self):
-        self._cache = None
+        self.__cache = None
         self.read = None
         self.readline = None
         self.readlines = None
@@ -492,18 +530,54 @@ class seek_wrapper:
         self.wrapped = None
 
 class eoffile:
-    # file-like object that always claims to be at end-of-file
+    # file-like object that always claims to be at end-of-file...
     def read(self, size=-1): return ""
     def readline(self, size=-1): return ""
+    # ...and also supports these response methods
+    def info(self):
+        return self.headers
+    def geturl(self):
+        return self.url
+
 
 class response_seek_wrapper(seek_wrapper):
-    # XXX this is tampering with the wrapped class :-((
+    """Avoids unnecessarily clobbering urllib.addinfourl methods on .close().
+
+    After .close():
+    , the following methods are supported:
+
+    .read()
+    .readline()
+    .readlines()
+    .seek()
+    .tell()
+    .info()
+    .geturl()
+    .__iter__()
+    .next()
+
+    Also supports pickling.
+
+    """
 
-    # Implementation inheritance: this is NOT a proper subclass of
-    # seek_wrapper.
-    # This class can only wrap response objects of urllib2.  It closes
-    # the socket while avoiding unnecessarily clobbering methods.
     def close(self):
-        if not isinstance(self.wrapped.fp, eoffile):
-            self.wrapped.fp.close()
-            self.wrapped.fp = eoffile()
+        self.headers = self.wrapped.headers
+        self.url = self.wrapped.url
+        self.wrapped.close()
+        self.wrapped = eoffile()
+
+    def __getstate__(self):
+        # There are three obvious options here:
+        # 1. truncate
+        # 2. read to end
+        # 3. close socket, pickle state including read position, then open
+        #    again on unpickle and use Range header
+
+        # 2 breaks pickle protocol, because one expects the original object
+        # to be left unscathed by pickling.  3 is too complicated and
+        # surprising (and too much work ;-) to happen in a sane __getstate__.
+        # So we do 1.
+
+        state = self.__dict__.copy()
+        state["wrapped"] = eoffile()
+        return state
