@@ -1,4 +1,4 @@
-"""Tests for ClientCookie._urllib2_support."""
+"""Tests for ClientCookie._urllib2_support (and for urllib2)."""
 
 # XXX
 # Request (I'm too lazy)
@@ -26,6 +26,10 @@ except NameError:
     True = 1
     False = 0
 
+## from ClientCookie import getLogger, DEBUG
+## l = getLogger("ClientCookie")
+## l.setLevel(DEBUG)
+
 class MockOpener:
     addheaders = []
     def open(self, req, data=None):
@@ -42,7 +46,8 @@ class MockHeaders(UserDict.UserDict):
     def getallmatchingheaders(self, name):
         r = []
         for k, v in self.data.items():
-            r.append("%s: %s" % (k, v))
+            if k.lower() == name:
+                r.append("%s: %s" % (k, v))
         return r
 
 class MockResponse(StringIO.StringIO):
@@ -149,7 +154,6 @@ class OpenerDirectorTests(unittest.TestCase):
             self.assert_(args == (req,))
 
     def test_handler_order(self):
-        from ClientCookie._urllib2_support import methnames
         o = OpenerDirector()
         handlers = []
         for meths, handler_order in [
@@ -249,6 +253,15 @@ class OpenerDirectorTests(unittest.TestCase):
                              isinstance(args[1], MockResponse))
 
 
+class MockHTTPResponse:
+    def __init__(self, fp, msg, status, reason):
+        self.fp = fp
+        self.msg = msg
+        self.status = status
+        self.reason = reason
+    def read(self):
+        return ''
+
 class MockHTTPClass:
     def __init__(self):
         self.req_headers = []
@@ -257,19 +270,19 @@ class MockHTTPClass:
     def __call__(self, host):
         self.host = host
         return self
-    def set_debuglevel(self, level): self.level = level
-    def putrequest(self, method, selector):
-        self.method, self.selector = method, selector
-    def putheader(self, key, value):
-        self.req_headers.append((key, value))
-    def endheaders(self):
+    def set_debuglevel(self, level):
+        self.level = level
+    def request(self, method, url, body=None, headers={}):
+        self.method = method
+        self.selector = url
+        self.req_headers.extend(headers.items())
+        if body:
+            self.data = body
         if self.raise_on_endheaders:
             import socket
             raise socket.error()
-    def send(self, data): self.data = data
-    def getreply(self): return 200, "OK", {}
-    def getfile(self): return MockFile()
-
+    def getresponse(self):
+        return MockHTTPResponse(MockFile(), {}, 200, "OK")
 
 class MockFTPWrapper:
     def __init__(self, data): self.data = data
@@ -432,98 +445,6 @@ class HandlerTests(unittest.TestCase):
                     self.assert_(o.req is req)
                     self.assertEqual(req.type, "ftp")
 
-##         def test_file(self):
-##             import time, rfc822, socket
-##             h = urllib2.FileHandler()
-##             o = h.parent = MockOpener()
-
-##             #from test_support import TESTFN
-##             TESTFN = "test.txt"
-##             towrite = "hello, world\n"
-##             for url in [
-##                 "file://localhost%s/%s" % (os.getcwd(), TESTFN),
-##                 "file://%s/%s" % (os.getcwd(), TESTFN),
-##                 "file://%s%s/%s" % (socket.gethostbyname('localhost'),
-##                                     os.getcwd(), TESTFN),
-##                 "file://%s%s/%s" % (socket.gethostbyname(socket.gethostname()),
-##                                     os.getcwd(), TESTFN),
-##                 # XXX Windows / Mac format(s), ... ?
-##                 ]:
-##                 create_time = time.time()
-##                 f = open(TESTFN, "w")
-##                 try:
-##                     try:
-##                         f.write(towrite)
-##                     finally:
-##                         f.close()
-
-##                     r = h.file_open(Request(url))
-##                     try:
-##                         data = r.read()
-##                         headers = r.info()
-##                         newurl = r.geturl()
-##                     finally:
-##                         r.close()
-##                     stats = os.stat(TESTFN)
-##                     modified = rfc822.formatdate(stats.st_mtime)
-##                 finally:
-##                     os.remove(TESTFN)
-##                 self.assert_(data == towrite)
-##                 self.assert_(headers["Content-type"] == "text/plain")
-##                 self.assert_(headers["Content-length"] == "13")
-## ##                 # Fudge Last-modified string comparison by one second to
-## ##                 # prevent spurious failure on crossing a second boundary while
-## ##                 # executing this test.
-## ##                 # XXXX This test still fails occasionally!  Why?
-## ##                 unfudged = rfc822.formatdate(create_time)
-## ##                 fudged = rfc822.formatdate(create_time+1)
-## ##                 self.assert_(headers["Last-modified"] in [unfudged, fudged])
-##                 self.assert_(headers["Last-modified"] == modified)
-
-##             for url in [
-##                 "file://localhost:80%s/%s" % (os.getcwd(), TESTFN),
-##                 # XXXX bug: these fail with socket.gaierror, should be URLError
-##                 #"file://%s:80%s/%s" % (socket.gethostbyname('localhost'),
-##                 #                       os.getcwd(), TESTFN),
-##                 #"file://somerandomhost.ontheinternet.com%s/%s" %
-##                 #(os.getcwd(), TESTFN),
-##                 ]:
-##                 try:
-##                     f = open(TESTFN, "w")
-##                     try:
-##                         f.write(towrite)
-##                     finally:
-##                         f.close()
-
-##                     self.assertRaises(urllib2.URLError,
-##                                       h.file_open, Request(url))
-##                 finally:
-##                     os.remove(TESTFN)
-
-##             h = urllib2.FileHandler()
-##             o = h.parent = MockOpener()
-##             # XXXX why does // mean ftp (and /// mean not ftp!), and where
-##             #  is file: scheme specified?  I think this is really a bug, and
-##             #  what was intended was to distinguish between URLs like:
-##             # file:/blah.txt (a file)
-##             # file://localhost/blah.txt (a file)
-##             # file:///blah.txt (a file)
-##             # file://ftp.example.com/blah.txt (an ftp URL)
-##             for url, ftp in [
-##                 ("file://ftp.example.com//foo.txt", True),
-##                 ("file://ftp.example.com///foo.txt", False),
-##                 # XXXX bug: fails with OSError, should be URLError
-##                 ("file://ftp.example.com/foo.txt", False),
-##                 ]:
-##                 req = Request(url)
-##                 try:
-##                     h.file_open(req)
-##                 except (urllib2.URLError, OSError):  # XXXX remove OSError
-##                     self.assert_(not ftp)
-##                 else:
-##                     self.assert_(o.req is req)
-##                     self.assert_(req.type == "ftp")
-
     def test_http(self):
         h = AbstractHTTPHandler()
         o = h.parent = MockOpener()
@@ -548,7 +469,9 @@ class HandlerTests(unittest.TestCase):
             self.assert_(http.method == method)
             self.assert_(http.selector == "/")
             http.req_headers.sort()
-            self.assert_(http.req_headers == [("Foo", "bar"), ("Spam", "eggs")])
+            self.assert_(http.req_headers == [
+                ("Connection", "close"),
+                ("Foo", "bar"), ("Spam", "eggs")])
             self.assert_(http.data == data)
 
         # check socket.error converted to URLError
@@ -565,7 +488,9 @@ class HandlerTests(unittest.TestCase):
                 self.assert_(not req.unredirected_hdrs.has_key("Content-length"))
                 self.assert_(not req.unredirected_hdrs.has_key("Content-type"))
             else:  # POST
-                self.assert_(req.unredirected_hdrs["Content-length"] == "0")
+                # No longer true, due to workarouhd for buggy httplib
+                # in Python versions < 2.4:
+                #self.assert_(req.unredirected_hdrs["Content-length"] == "0")
                 self.assert_(req.unredirected_hdrs["Content-type"] ==
                              "application/x-www-form-urlencoded")
             # XXX the details of Host could be better tested
@@ -584,15 +509,24 @@ class HandlerTests(unittest.TestCase):
             self.assert_(req.unredirected_hdrs["Spam"] == "foo")
 
     def test_request_upgrade(self):
+        new_req_class = hasattr(urllib2.Request, "has_header")
+
         h = HTTPRequestUpgradeProcessor()
         o = h.parent = MockOpener()
 
-        # urllib2.Request gets upgraded
+        # urllib2.Request gets upgraded, unless it's the new Request
+        # class from 2.4
         req = urllib2.Request("http://example.com/")
         newreq = h.http_request(req)
-        self.assert_(newreq is not req)
-        self.assert_(newreq.__class__ is Request)
-        # ClientCookie._urllib2_support.Request doesn't
+        if new_req_class:
+            self.assert_(newreq is req)
+        else:
+            self.assert_(newreq is not req)
+        if new_req_class:
+            self.assert_(newreq.__class__ is not Request)
+        else:
+            self.assert_(newreq.__class__ is Request)
+        # ClientCookie._urllib2_support.Request doesn't get upgraded
         req = Request("http://example.com/")
         newreq = h.http_request(req)
         self.assert_(newreq is req)
@@ -697,13 +631,24 @@ class HandlerTests(unittest.TestCase):
             "read",
             ("can_fetch", "", url),
             ])
+        # https url -> should fetch robots.txt from https url too
+        rfpc.clear()
+        url = "https://example.org/rhubarb.html"
+        req = Request(url)
+        h.http_request(req)
+        self.assert_(rfpc.calls == [
+            "__call__",
+            ("set_url", "https://example.org/robots.txt"),
+            "read",
+            ("can_fetch", "", url),
+            ])
 
     def test_cookies(self):
         cj = MockCookieJar()
         h = HTTPCookieProcessor(cj)
         o = h.parent = MockOpener()
 
-        req = urllib2.Request("http://example.com/")
+        req = Request("http://example.com/")
         r = MockResponse(200, "OK", {}, "")
         newreq = h.http_request(req)
         self.assert_(cj.ach_req is req is newreq)
@@ -730,7 +675,7 @@ class HandlerTests(unittest.TestCase):
         o = h.parent = MockOpener()
 
         req = Request("http://example.com/")
-        r = MockResponse(200, "OK", {"Foo": "Bar"},
+        r = MockResponse(200, "OK", MockHeaders({"Foo": "Bar", "Content-type": "text/html"}),
                          '<html><head>'
                          '<meta http-equiv="Refresh" content="spam">'
                          '</head></html>')
@@ -740,16 +685,17 @@ class HandlerTests(unittest.TestCase):
         self.assert_(headers["Foo"] == "Bar")
 
     def test_refresh(self):
-        # XXX processor constructor optional args
-        h = HTTPRefreshProcessor()
-        o = h.parent = MockOpener()
+        # XXX test processor constructor optional args
+        h = HTTPRefreshProcessor(max_time=None, honor_time=False)
 
-        req = Request("http://example.com/")
-        headers = MockHeaders({"refresh": '0; url="http://example.com/foo/"'})
-        r = MockResponse(200, "OK", headers, "")
-        newr = h.http_response(req, r)
-        self.assert_(o.proto == "http")
-        self.assert_(o.args == (req, r, "refresh", "OK", headers))
+        for val in ['0; url="http://example.com/foo/"', "2"]:
+            o = h.parent = MockOpener()
+            req = Request("http://example.com/")
+            headers = MockHeaders({"refresh": val})
+            r = MockResponse(200, "OK", headers, "")
+            newr = h.http_response(req, r)
+            self.assertEqual(o.proto, "http")
+            self.assertEqual(o.args, (req, r, "refresh", "OK", headers))
 
     def test_redirect(self):
         from_url = "http://example.com/a.html"
@@ -815,6 +761,7 @@ class HeadParserTests(unittest.TestCase):
 
     def test(self):
         # XXX XHTML
+        from ClientCookie import HeadParser
         htmls = [
             ("""<meta http-equiv="refresh" content="1; http://example.com/">
             """,
@@ -832,7 +779,42 @@ class HeadParserTests(unittest.TestCase):
              [("refresh", "1; http://example.com/"), ("foo", "bar")])
             ]
         for html, result in htmls:
-            self.assert_(parse_head(StringIO.StringIO(html)) == result)
+            self.assertEqual(parse_head(StringIO.StringIO(html), HeadParser()), result)
+
+
+class MockHTTPHandler(HTTPHandler):
+    def __init__(self): self._count = 0
+    def http_open(self, req):
+        import mimetools
+        from StringIO import StringIO
+        if self._count == 0:
+            self._count = self._count + 1
+            msg = mimetools.Message(
+                StringIO("Location: http://www.cracker.com/\r\n\r\n"))
+            return self.parent.error(
+                "http", req, MockFile(), 302, "Found", msg)
+        else:
+            self.req = req
+            msg = mimetools.Message(StringIO("\r\n\r\n"))
+            return MockResponse(200, "OK", msg, "", req.get_full_url())
+
+class MiscTests(unittest.TestCase):
+
+    def test_cookie_redirect(self):
+        # cookies shouldn't leak into redirected requests
+        from ClientCookie import CookieJar, build_opener, HTTPHandler, \
+             HTTPCookieProcessor
+        from urllib2 import HTTPError
+
+        from test_cookies import interact_netscape
+
+        cj = CookieJar()
+        interact_netscape(cj, "http://www.example.com/", "spam=eggs")
+        hh = MockHTTPHandler()
+        cp = HTTPCookieProcessor(cj)
+        o = build_opener(hh, cp)
+        o.open("http://www.example.com/")
+        self.assert_(not hh.req.has_header("Cookie"))
 
 
 class MyHTTPHandler(HTTPHandler): pass
@@ -885,7 +867,7 @@ class FunctionTests(unittest.TestCase):
             self.assert_(False)
 
     def _methnames(self, *objs):
-        from ClientCookie._urllib2_support import methnames
+        from ClientCookie._Opener import methnames
         r = []
         for i in range(len(objs)):
             obj = objs[i]
